/*
 * Copyright (c) 2011 Wolfgang Solfrank.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the  
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *      This product includes software developed by Wolfgang Solfrank.
 * 4. The name of TooLs GmbH may not be used to endorse or promote products
 *    derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY TOOLS GMBH ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL TOOLS GMBH BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
 * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#ifndef	__amd64_pte_h__
#define	__amd64_pte_h__

#ifndef	__ASSEMBLY__
/*
 * Readonly access to page directory.
 */
extern uint64_t rdcr3(void);

/*
 * Format of amd64 page table (directory) entries.
 */
typedef	uint64_t	pte_t;
#endif

#define	PTE_NX		0x8000000000000000
#define	PTE_PADDR	0x000ffffffffff000
#define	PTE_GBL		0x0000000000000100
#define	PTE_PAT		0x0000000000000080
#define	PTE_DIRTY	0x0000000000000040
#define	PTE_ACC		0x0000000000000020
#define	PTE_PCD		0x0000000000000010
#define	PTE_PWT		0x0000000000000008
#define	PTE_USER	0x0000000000000004
#define	PTE_RW		0x0000000000000002
#define	PTE_PRSNT	0x0000000000000001

#define	PGLVLS		4
#define	PGLVLSHFT	9
#define	PGSHIFT		12
#define	PAGESIZE	(1 << PGSHIFT)
#define	PGLVLIDX	((1 << PGLVLSHFT) - 1)
#define	MASKBITS	(64 - PGLVLS * PGLVLSHFT - PGSHIFT)

/* The size of the virtual address space. */
#define	VIRT_SPACE	(1UL << (PGLVLS * PGLVLSHFT + PGSHIFT))

/* Mask off any unsupported address bits from a virtual address. */
#define	MASK_VADDR(va)	((uint64_t)(va) & (VIRT_SPACE - 1))

/* Extend the high bit of a virtual address. */
#define	UN_MASK_VADDR(va)	(((int64_t)(va) << MASKBITS) >> MASKBITS)

#define	PG_LVLIDX(lvl, va)	(MASK_VADDR(va) \
				 >> (PGSHIFT + ((lvl) - 1) * PGLVLSHFT))

/* We map (most) memory only every other page for red zone support. */
#define	MEM_STRIDE	2

#endif
