/*
 * Copyright (c) 2011 Wolfgang Solfrank.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the  
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *      This product includes software developed by Wolfgang Solfrank.
 * 4. The name of TooLs GmbH may not be used to endorse or promote products
 *    derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY TOOLS GMBH ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL TOOLS GMBH BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
 * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#ifndef	__FRAME_H__
#define	__FRAME_H__

/*
 * Various stack frames, as found in Xen.
 */
typedef	uint64_t register_t;

/* This is on the stack on entry to the failsafe callback: */
struct failsafe_frame {
	register_t	rcx;
	register_t	r11;
	register_t	ds;
	register_t	es;
	register_t	fs;
	register_t	gs;
	register_t	rip;
	register_t	cs;
	register_t	rflags;
	register_t	rsp;
	register_t	ss;
};

/* This is expected by HYPVERVISOR_iret: */
struct h_iret_frame {
	register_t	rax;
	register_t	r11;	/* Why is r11 & rcx exchanged here ? XXX */
	register_t	rcx;
	register_t	flags;
	register_t	rip;
	register_t	cs;
	register_t	rflags;
	register_t	rsp;
	register_t	ss;
};

/* This frame is expected by a hardware iret instruction: */
struct iret_frame {
	register_t	rip;
	register_t	cs;
	register_t	rflags;
	register_t	rsp;
	register_t	ss;
};

/* This frame is found on entry to the event handler: */
struct event_frame {
	register_t	rcx;
	register_t	r11;
	register_t	rip;
	register_t	cs;
	register_t	rflags;
	register_t	rsp;
	register_t	ss;
};

#endif
